// CompilerTest.cpp : This file contains the 'main' function. Program execution begins and ends there.
//

#include <iostream>
#include <string>

long Utf8Len(const char* const utf, size_t size = 0)
{
	// returns the length of the string or -1 if the integrity scan fails
	unsigned long i, count;
	unsigned char c, d;

	//  Scan the UTF-8 string, verifying its integrity and discovering its character count.
	//  Note:  This integrity check does not detect characters encoded with more bytes than
	//         necessary.

	if (utf == nullptr)
		return 0;

	i = 0;
	count = 0;
	if (size == 0)
		size = strlen(utf);

	c = utf[i++];

	while (i <= size)
	{
		if ((++count) > 2000000000)
			return -1;   //  (more than 2 billion characters assumed to be an error)

		if (c & 128)
		{
			if (!(c & 64))
				return -1;

			d = utf[i++];
			if ((!(d & 128)) || (d & 64))
				return -1;

			if (c & 32)
			{
				d = utf[i++];
				if ((!(d & 128)) || (d & 64))
					return -1;

				if (c & 16)
				{
					d = utf[i++];
					if ((!(d & 128)) || (d & 64) || (c & 8))
						return -1;
				}
			}
		}
		c = utf[i++];
	}
	return count;
}

long Utf8Len(short* const wordStr)
{
	if (wordStr == 0)
		return 0;

	unsigned long i = 0, j = 0;
	short c = wordStr[i++];

	while (c && (j < 2000000000))
	{
		if (c < 128)
			j++;
		else if (c < 2048)
			j += 2;
		else
			j += 3;

		c = wordStr[i++];
	}
	return j;
}

void WordStrToUtf8(short* wordStr, size_t wLength, char* utf8, size_t uLength)
{
	if ((wordStr == nullptr) || (wLength == 0))
	{
		if (uLength > 0)
			*utf8 = 0;

		return;
	}

	size_t i = 0; // iterates over the word string
	size_t j = 0; // iterates over the UTF8 string

	short c = wordStr[i];
	unsigned char* utf = (unsigned char*)utf8;

	while (c && (i < wLength) && (j < uLength))
	{
		if (c < 128)
		{
			utf[j++] = static_cast<unsigned char>(c);
		}
		else if (c < 2048)
		{
			utf[j++] = 192 + static_cast<unsigned char>(c / 64);
			utf[j++] = 128 + static_cast<unsigned char>(c & 63);
		}
		else
		{
			utf[j++] = 224 + static_cast<unsigned char>(c / 4096);
			utf[j++] = 128 + static_cast<unsigned char>((c & 4095) / 64);
			utf[j++] = 128 + static_cast<unsigned char>(c & 63);
		}
		c = wordStr[++i];
	}

	// and set the terminating zero byte
	utf[j] = 0;
}

std::string StringAsUTF8(const char* const C)
{
	if (!C)
		return "";

	long len = Utf8Len(C);
	if (len >= 0)
		return std::string(C);

	size_t wlen = strlen(C);
	if (wlen == 0)
		return "";

	wchar_t* str = new wchar_t[wlen + 1];
	const char* s = C;
	for (size_t n = 0; n <= wlen; n++)
	{
		str[n] = (unsigned char)s[n];
	}

	long utflen = Utf8Len((short*)str);
	char* utf8 = new char[utflen + 1];

	WordStrToUtf8((short*)str, wlen, utf8, utflen);

	std::string S(utf8);

	delete[] utf8;
	delete[] str;

	return S;
}

int main()
{
	std::string s = "<b>{BC}</b><br>{volume|l|---|f}";
	std::string utf = StringAsUTF8(s.c_str());

    std::cout << utf << "\n";
}

